/******************************************************************************
 *
 * AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: 
 * Simon Vey, Thomas Witkowski, Andreas Naumann, Simon Praetorius, et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * This file is part of AMDiS
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/



/** \file GlobalDOFNumbering.h */

#ifndef AMDIS_GLOBALDOFNUMBERING_H
#define AMDIS_GLOBALDOFNUMBERING_H

#include <map>
#include <vector>
#include "Global.h"
#include "AMDiS_fwd.h"

namespace AMDiS {

  /** \ingroup Parallel
   * \brief 
   * Creates a global DOF numbering.
   */
  class GlobalDOFNumbering
  {
  public:
    /// Constructor
    GlobalDOFNumbering(std::vector<MeshStructure*> &meshCodes,
		       std::vector< std::vector<DegreeOfFreedom> > &dofCodes,
		       int dofsPerElement);

    /// Returns the local index of rank corresponding to global index.
    DegreeOfFreedom getLocalIndex(int rank, DegreeOfFreedom globalIndex);

    /// Returns the global index corresponing to local index of rank.
    DegreeOfFreedom getGlobalIndex(int rank, DegreeOfFreedom localIndex);

  protected:
    void handleRecursive(std::vector<MeshStructure*> &meshCodes,
			 std::vector< std::vector<DegreeOfFreedom>::iterator> &it);

  protected:
    std::vector< std::map<DegreeOfFreedom, DegreeOfFreedom> > localToGlobal_;

    std::vector< std::map<DegreeOfFreedom, DegreeOfFreedom> > globalToLocal_;

    int dofsPerElement_;

    DegreeOfFreedom globalIndexCounter_;
  };

}

#endif
